import os
from dotenv import load_dotenv
from pathlib import Path

ROOT_DIR = str(Path(os.path.abspath(__file__)).parent.parent)

if 'DBFILE' not in os.environ:
    import warnings
    warnings.warn(
        "Environment variables are not specified! Manually load from `.env` file"
    )
    envfile = os.path.join(ROOT_DIR, ".env")
    load_dotenv(envfile)

DBFILE = os.environ['DBFILE']
if DBFILE.startswith("."):
    # relative path, we need to join between the current one
    DBFILE = os.path.abspath(os.path.join(ROOT_DIR, DBFILE))
else:
    DBFILE = os.path.abspath(DBFILE)

CONFIG_FILE = os.environ.get('CONFIG', os.path.join(ROOT_DIR, "experiments.yml"))



import math
from operator import attrgetter

import seaborn as sns
import streamlit as st
from streamlit.callbacks.callbacks import periodic

from osin.config import CONFIG_FILE
from osin.db import ExpResult, Job, db
from osin.exp_config import ExpConfig
from osin.ui.toggle_list import toggle_list

# apply general settings
st.set_page_config(layout="wide")
sns.set_theme()
containers = {'jobs': None, 'exps': []}


@st.cache(suppress_st_warning=True, allow_output_mutation=True)
def get_session():
    return {}


session = get_session()


class Dashboard:

    @staticmethod
    def update():
        Dashboard.render_running_jobs()

    @staticmethod
    def render_running_jobs(force: bool = False):
        global containers, session
        if containers['jobs'] is None:
            return

        with containers['jobs']:
            should_rerun = True

            with db.atomic():
                last_finished_job_id = Job.last_finished_job()
                # get the next job
                next_jobs = list(Job.select(Job.id, Job.status).where(Job.id == last_finished_job_id + 1))
                if len(next_jobs) == 0:
                    # key will be the latest job
                    key = f"{last_finished_job_id}:finished"
                else:
                    # key would be the next job & status
                    # when the job is finish or has status updated, this key will change
                    key = f"{next_jobs[0].id}:{next_jobs[0].status}"

                if not force and session.get("render_running_jobs_key", None) == key:
                    should_rerun = False

                if should_rerun:
                    # retrieve the jobs
                    start = max(last_finished_job_id - 3, 0)
                    end = last_finished_job_id + 5
                    jobs = Job.select().where((Job.id >= start) & (Job.id < end))
                    jobs = [
                        dict(id=job.id,
                             name=' '.join(job.exec_run_args),
                             icon={"queueing": "hourglass", "success": "circle-check", "started": "spinner",
                                   "failure": "triangle-exclaimation"}[job.status],
                             value=job.hostname)
                        for job in sorted(list(jobs), key=attrgetter('id'), reverse=True)
                    ]

            if should_rerun:
                with containers['jobs']:
                    session['render_running_jobs_key'] = key
                    toggle_list(jobs)


# periodically check if data has been updated
periodic(1.0, Dashboard.update)

# render experiments
exp_configs = ExpConfig.from_file(CONFIG_FILE)
for exp_config in exp_configs:
    st.markdown(f"# {exp_config.name}")

    report_grids = exp_config.report()
    for row in report_grids:
        col_sizes = [r['colspan'] for r in row]
        if sum(col_sizes) < 24:
            # add a padding column so that streamlit don't auto-scale the columns
            col_sizes.append(24 - sum(col_sizes))

        cols = st.beta_columns(col_sizes)
        for col, report in zip(cols, row):
            with col:
                if report['display_name']:
                    st.markdown(f"### {report['name']}")
                value = report['get_value']()
                st.write(value)

    with st.form(key='run exp'):
        # create a grid
        n_cols = 3
        n_rows = math.ceil(len(exp_config.parameters) / n_cols)
        params = list(exp_config.parameters.items())
        idx = 0
        selected_params = {}
        for i in range(n_rows):
            cols = st.beta_columns(n_cols)
            for col in cols:
                with col:
                    k, v = params[idx]
                    selected_params[k] = st.multiselect(label=k, options=v)
                    idx += 1

                    if idx >= len(params):
                        break
        run_exp = st.form_submit_button(label='Run experiment')

    if run_exp:
        missing_params = [k for k, v in selected_params.items() if len(v) == 0]
        if len(missing_params) > 0:
            st.error(f"Missing values for parameter: {missing_params}")
        else:
            jobs = exp_configs[0].trigger_runs(selected_params)
            st.write(f"Start {len(jobs)} jobs.\n")

    st.markdown(f"## Raw Data")
    containers['exps'].append(st.beta_container())
    with containers['exps'][-1]:
        st.write(ExpResult.as_dataframe(exp_config.table_name))

st.markdown(f"## Running Jobs")
containers['jobs'] = st.empty()
Dashboard.render_running_jobs(force=True)



import os
import subprocess
import time

import click
from loguru import logger

parent = os.path.dirname(os.path.abspath(__file__))


@click.command()
@click.option("--pid_file", "-p", default="/tmp/osin.pid", help="File contains child processes' id")
@click.option("--grace_period", "-t", default=30, type=int, help="Maximum time waiting to stop the process. Default 30 seconds")
def main(pid_file: str, grace_period: int):
    if os.path.exists(pid_file):
        logger.error("Previous processes are still running. Stop them first!")
        return

    env = dict(os.environ)
    logger.info("Start streamlit...")
    p1 = subprocess.Popen(["streamlit", "run", os.path.join(parent, "ui/dashboard.py")], env=env)
    with open(pid_file, "a") as f:
        f.write(str(p1.pid))
        f.write("\n")

    logger.info("Start worker...")
    p2 = subprocess.Popen(["python", "-m", "osin.worker"], env=env)
    with open(pid_file, "a") as f:
        f.write(str(p2.pid))
        f.write("\n")

    logger.info("Start server...")
    p3 = subprocess.Popen(["python", "-m", "osin.server"], env=env)
    with open(pid_file, "a") as f:
        f.write(str(p3.pid))
        f.write("\n")

    try:
        p1.wait()
        p2.wait()
        p3.wait()
    except KeyboardInterrupt:
        logger.info("Receive termination signal. Stop the application")
        p1.terminate()
        p2.terminate()
        p3.terminate()

    print("Wait for the application to fully stop", end="", flush=True)
    for i in range(grace_period):
        if any(p.poll() is None for p in [p1, p2, p3]):
            print(".", end="", flush=True)
            time.sleep(1)

    if all(p.returncode is not None for p in [p1, p2, p3]):
        logger.info("Terminate the application successfully! Return code: {}", [p.returncode for p in [p1, p2, p3]])
        os.remove(pid_file)
    else:
        logger.error("We can't fully stop the application")
        print("Return code:", [p.returncode for p in [p1, p2, p3]])


if __name__ == '__main__':
    main()



from flask import Flask, request, jsonify

from osin.db import ExpResult, Job

app = Flask(__name__)


@app.route("/api/v1/runs", methods=['POST'])
def save_run():
    ExpResult.create(table=request.json['table'], data=request.json['data'])
    jobs = Job.select().where(Job.hostname == request.json['hostname'], Job.pid == str(request.json['pid']))
    jobs = list(jobs)
    if len(jobs) > 0:
        # ignore when users submit an orphan job, mark the job that we finished
        assert len(jobs) == 1, "We have multiple jobs of samse hostname and process id. This shouldn't happen"
        jobs[0].status = "success"
        jobs[0].save()
    return jsonify({"status": "success"}), 200


@app.route("/api/v1/runs/error", methods=['POST'])
def run_error():
    jobs = Job.select().where(
        Job.hostname == request.json['hostname'], Job.pid == str(request.json['pid']))
    jobs = list(jobs)
    if len(jobs) > 0:
        # ignore when users submit an orphan job, mark the job that we finished
        assert len(jobs) == 1, "We have multiple jobs of same hostname and process id. This shouldn't happen"
        jobs[0].status = "failure"
        jobs[0].save()
    return jsonify({"status": "success"}), 200


if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5524)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['osin', 'osin.ui', 'osin.ui.toggle_list']

package_data = \
{'': ['*'],
 'osin.ui.toggle_list': ['frontend/*',
                         'frontend/build/*',
                         'frontend/build/static/js/*',
                         'frontend/public/*',
                         'frontend/src/*']}

install_requires = \
['Flask>=2.0.1,<3.0.0',
 'loguru>=0.5.3,<0.6.0',
 'matplotlib>=3.4.2,<4.0.0',
 'pandas>=1.2.4,<2.0.0',
 'peewee>=3.14.4,<4.0.0',
 'plotly>=5.0.0,<6.0.0',
 'python-dotenv>=0.17.1,<0.18.0',
 'ruamel.yaml>=0.17.9,<0.18.0',
 'seaborn>=0.11.1,<0.12.0',
 'streamlit-callbacks>=0.0.5,<0.0.6',
 'streamlit>=0.82.0,<0.83.0']

setup_kwargs = {
    'name': 'osin',
    'version': '0.1.4',
    'description': 'Rethink Experimenting',
    'long_description': None,
    'author': 'Binh Vu',
    'author_email': 'binh@toan2.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/binh-vu/osin',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
